@@ -8,7 +8,7 @@ use Symbol;
 use Carp;
 use Fcntl qw(F_GETFL O_WRONLY O_RDWR O_APPEND);
 
-our $VERSION = "0.08";
+our $VERSION = "0.09";
 
 use base 'Exporter';
 
@@ -66,6 +66,7 @@ sub encode {
 
     if (!ref($value)) {
         my $ref_to_value = \$value;
+        my $refaddr     = Scalar::Util::refaddr($ref_to_value);
         my $ref = ref($ref_to_value);
         my $encoded_value = $value;
         # perl 5.8 - ref() with a vstring returns SCALAR
@@ -75,7 +76,7 @@ sub encode {
         ) {
             $encoded_value = encode($ref_to_value, $path_expr, $seen);
             delete $encoded_value->{__refaddr};
-            delete $seen->{$ref_to_value};
+            delete $seen->{$refaddr};
         }
         return $encoded_value;
     }
@@ -89,15 +90,15 @@ sub encode {
 
     my $encoded_value;
 
-    if ($seen->{$value}) {
+    if ($seen->{$refaddr}) {
         $encoded_value = {  __reftype => $reftype,
                             __refaddr => $refaddr,
                             __recursive => 1,
-                            __value => $seen->{$value} };
+                            __value => $seen->{$refaddr} };
         $encoded_value->{__blessed} = $blesstype if $blesstype;
         return $encoded_value;
     }
-    $seen->{$value} = $path_expr;
+    $seen->{$refaddr} = $path_expr;
 
     # Build a new path string for recursive calls
     my $_p = sub {
@@ -170,6 +171,13 @@ sub encode {
         $encoded_value = [ unpack('c*', $$value) ];
     } elsif ($reftype eq 'SCALAR') {
         $encoded_value = encode($$value, &$_p, $seen);
+
+    } elsif ($reftype eq 'IO') {
+        if ( $encoded_value->{IO} = encode(fileno($value), &$_p, $seen) )
+        {
+            $encoded_value->{IOmode} = _get_open_mode(*$value);
+            $encoded_value->{IOseek} = sysseek($value, 0, 1);
+        }
     }
 
     $encoded_value = { __reftype => $reftype, __refaddr => $refaddr, __value => $encoded_value };
@@ -286,15 +294,17 @@ sub _create_anon_ref_of_type {
     }
 }
 
+# $fh can be undef, in which case it's autovivified.  But for handles that
+# were originally created via Synbol::geniosym, it'll be passed in already
+# created
 sub _recreate_fh {
-    my($fileno, $mode) = @_;
+    my($fileno, $mode, $fh) = @_;
 
-    my $fh;
     if ($mode) {
         open($fh, $mode . '&=', $fileno)
             || Carp::carp("Couldn't open filehandle for descriptor $fileno with mode $mode: $!");
 
-    } else {
+    } elsif ($fileno) {
         open($fh, '>&=', $fileno)
         || open($fh, '<&=', $fileno)
         || Carp::carp("Couldn't open filehandle for descriptor $fileno: $!");
@@ -385,6 +395,10 @@ sub decode {
         my $vstring = eval 'v' . join('.', @$value);
         $rv = $refaddr ? \$vstring : $vstring;
 
+    } elsif ($reftype eq 'IO') {
+        # A filehandle that was created via Symbol::geniosym
+        my $fh = Symbol::geniosym;
+        $rv = _recreate_fh($value->{IO}, $value->{IOmode}, $fh);
     }
 
     bless $rv, $blessed if ($blessed and ! $input->{__recursive});
@@ -436,6 +450,8 @@ sub _validate_decode_structure {
                 or
                 ( $reftype eq 'VSTRING' and ref($value) eq 'ARRAY' )
                 or
+                ( $reftype eq 'IO' and exists($value->{IO}) )
+                or
                 ( $reftype and ! ref($input->{__value}) and $input->{__recursive} )
                 or
                 ( $input->{__tied} and ref($input->{__value}) and $input->{__value}->{__blessed} )
